# -*- coding: utf-8 -*-
import copy
import json

import os.path
from ecimporter.lib.config import Participant, Outputindicator, Ignored


class ConfigValueError(Exception):
    """This error shows that, the bearbeiter field value is not excepted.
    We use the bearbeiter id to map to uid and gid."""
    pass


class WrongStateError(Exception):
    """This error is thrown, if the input dict is not 7 and accepted."""
    pass


class DictExtractor(object):
    def __init__(self, dest_prototype, mapping=None):
        self.dest_prototype = dest_prototype
        self.mapping = mapping or {}

    def extract(self, src_dict):
        result_dict = copy.deepcopy(self.dest_prototype)
        for key in src_dict:
            if key in result_dict and not key in self.mapping.values():
                result_dict[key] = src_dict[key]
            if key in self.mapping:
                result_dict[self.mapping[key]] = src_dict[key]
        return result_dict


def filter_ba_list(list1):
    """Filter list of ba dicts which have application_state_id 7.
    (Abgeschlossen&Angenommen)
    """

    def helper(data):
        return (str(data.get("application_state_id")) == "7" and
                data.get("vertragsabschluss"))

    return filter(helper, list1)


def entry(pt_dict, pn_dict, oi_dict):
    dict1 = {
        "participant": pt_dict,
        "participation": pn_dict,
        "oi": oi_dict,
    }
    return dict1


def map_field(list_, mapping, dfield, vfield):
    def helper(data):
        old = data.get(dfield)
        if dfield in data:
            data[vfield] = mapping.get(data[dfield])
        else:
            raise ConfigValueError("No %s in appliciant with personal_id %s"
                                   % (dfield, str(data.get('personal_id'))))

        if data.get(vfield) is None:
            raise ConfigValueError(("In %s mapping not found %s in appliciant "
                                    "with personal_id %s")
                                   % (dfield, old,
                                      str(data.get('personal_id'))))

        return data

    return map(helper, list_)


def transfer_model(json_str, bearbeiter_to_uid, bearbeiter_to_gid):
    full_list = json.loads(json_str)
    list_ = filter_ba_list(full_list)
    list_ = map_field(list_, bearbeiter_to_uid, "bearbeiter", "uid")
    list_ = map_field(list_, bearbeiter_to_gid, "bearbeiter", "gid")
    participants_list = []
    participant_de = DictExtractor(Participant)
    participation_de = DictExtractor({"uid": None, "gid": None,
                                      "start_date": None},
                                     {"vertragsabschluss": "start_date",
                                      "course_end_date": "planned_end_date"})
    oi_de = DictExtractor(Outputindicator,
                          {"address_federalstate": "person_federal_state"})
    if len(list_) != 1:
        raise WrongStateError("The input dict is not state 7 or doesnt have "
                              "vertragsabschluss date.")
    for data in list_:
        participants_list.append(entry(participant_de.extract(data),
                                       participation_de.extract(data),
                                       oi_de.extract(data)))
    return participants_list


def get_editor_mapping(request):
    config_file_path = request.registry.settings.get(
        "ecimporter.bearbeiter.maps")
    if not config_file_path or not os.path.exists(config_file_path):
        raise ConfigValueError("No bearbeiter map config found")
    config_dict = (None, None)
    with open(config_file_path) as config_file:
        config_dict = json.load(config_file)
    return config_dict["bearbeiter_to_uid"], config_dict["bearbeiter_to_gid"]
